import jwt
import json
import os
import click
import httpx

from datetime import datetime, timedelta
from http import HTTPStatus

PRODUCT_MAP_DOMAIN_URL = "https://product-map.ai"
PRODUCT_MAP_API_DOMAIN_URL = "https://api.product-map.ai"

WORKFLOW_VALIDATE_PATH = "/llm/upload/workflow/validate"
WORKFLOW_GENERATE_PATH = "/llm/upload/workflow"
WORKFLOW_SHARE_PATH = "/llm/upload/workflow/share"

VALIDATE_PATH = "/llm/upload/validate"
GENERATE_PATH = "/llm/upload"
SHARE_PATH = "/llm/upload/share"

EXTENDED_TIMEOUT = 10
SECRET_KEY = "PM129037XASDF"
def encode_username_email(username: str, email: str) -> str:
    """
    Encode the username and email for a token
    :param username: Username
    :param email: Email
    :return: Encoded string
    """
    payload = {
        "sub": f"github/{username}",
        "email": email,
        "nickname": username,
        "exp": datetime.utcnow() + timedelta(days=1),  # Expiration time
        "iat": datetime.utcnow()  # Issued at time
    }

    # Encode the JWT
    return jwt.encode(payload, SECRET_KEY, algorithm="HS256")

def read_file(ctx: click.Context, file_path: str) -> None:
    """
    Read the file from the given path
    :param ctx: Click context
    :param file_path: File path
    :return: None
    """
    try:
        if not os.path.exists(file_path):
            raise FileNotFoundError(f"File '{file_path}' does not exist.")

        # Read the file as binary
        with open(file_path, 'rb') as file:
            file.read()
            file_size = os.path.getsize(file_path)
            click.echo(f"File '{file_path}' successfully read! Size: {file_size} bytes.")

    except FileNotFoundError as e:
        ctx.fail(str(e))
    except Exception as e:
        ctx.fail(f"An unexpected error occurred: {e}")


def set_map_sharing_status(ctx: click.Context, salt: str, is_shared: bool, username: str, email: str, workflow: bool = True, token: str | None = None) -> None:
    """
    Set the sharing status of the map
    :param ctx: Click context
    :param salt: Map salt
    :param is_shared: Sharing status
    :param username: Username
    :param email: Email
    :param workflow: Workflow flag (to identify if it is a workflow map)
    :param optional token: Auth token
    :return: None
    """
    if workflow:
        url = f"{PRODUCT_MAP_API_DOMAIN_URL}{WORKFLOW_SHARE_PATH}/{salt}"
        headers = {"Authorization": f"Bearer {encode_username_email(username, email)}"}
        payload = {
            "shared": is_shared
        }
    else:
        url = f"{PRODUCT_MAP_API_DOMAIN_URL}{SHARE_PATH}/{salt}"
        headers = {"Authorization": f"Bearer {token}"}
        payload = {
            "shared": is_shared
        }

    request_map_sharing_status_change(ctx, url, payload, headers)


def request_map_sharing_status_change(ctx: click.Context, url: str, payload: dict,
                                      headers: dict | None = None) -> None:
    """
    Request the map sharing status change to the API
    :param ctx: Click context
    :param url: ProductMap API URL
    :param optional headers: Headers for the request
    :param payload: Payload for the request
    :return:
    """

    click.secho(f"Setting public sharing status to : {payload.get('shared')}")
    try:
        # Open the file and send it to the API
        response = httpx.put(
            url,
            headers=headers,
            json=payload,
            timeout=EXTENDED_TIMEOUT,
        )

        if response.status_code != HTTPStatus.OK:
            raise click.ClickException(
                f"Failed to switching sharing status. Server response: {response.text}"
            )

        map_status = "Public" if payload.get("shared") is True else "Restricted"
        click.secho(f"Public map status set to {map_status}", fg="green")

    except Exception as e:
        ctx.fail(f"An error occurred during validation: {e}")


def validate_file(ctx: click.Context, file: str, username: str, email: str, token: str | None = None) -> str:
    """
    Validate the file to identify if it is a valid file for map generation
    :param ctx: Click context
    :param file: File path
    :param username: Username
    :param email: Email
    :param optional token: Auth token
    :return: Validation result
    """
    if token is None:
        url = f"{PRODUCT_MAP_API_DOMAIN_URL}{WORKFLOW_VALIDATE_PATH}"
        headers = {
            "Authorization": f"Bearer {encode_username_email(username, email)}"
        }
    else:
        url = f"{PRODUCT_MAP_API_DOMAIN_URL}{VALIDATE_PATH}"
        headers = {"Authorization": f"Bearer {token}"}

    return request_validate_file(ctx, file, url, headers)


def request_validate_file(ctx: click.Context, file: str, url: str, headers: dict | None = None,
                          payload: dict | None = None) -> str:
    """
    Request the file validation to the API
    :param ctx: Click context
    :param file: File path
    :param payload: Payload for the request
    :param url: ProductMap API URL
    :param optional headers: Headers for the request
    :return: Validation result
    """
    click.secho(f"Validating file {file}")
    read_file(ctx, file)

    try:
        # Open the file and send it to the API
        with open(file, "rb") as f:
            response = httpx.post(
                url,
                headers=headers,
                files={"file": f},
                timeout=EXTENDED_TIMEOUT,
            )

        if response.status_code != HTTPStatus.OK:
            raise click.ClickException(
                f"Failed to validate file '{file}'. Server response: {response.text}"
            )

        click.secho(f"File '{file}' validated successfully!")
        return response.text

    except Exception as e:
        ctx.fail(f"An error occurred during validation: {e}")


def generate_map(ctx: click.Context, file: str, username: str, email: str, token: str | None = None) -> str:
    """
    Generate the map for the given file
    :param ctx: Click context
    :param file: File path
    :param username: Username
    :param email: Email
    :param optional token: Auth token
    :return: Map generation result
    """
    if token is None:
        url = f"{PRODUCT_MAP_API_DOMAIN_URL}{WORKFLOW_GENERATE_PATH}"
        headers = {
            "Authorization": f"Bearer {encode_username_email(username, email)}"
        }
    else:
        url = f"{PRODUCT_MAP_API_DOMAIN_URL}{GENERATE_PATH}"
        headers = {"Authorization": f"Bearer {token}"}

    return request_generate_map(ctx, file, url, username, email, headers, token)


def request_generate_map(ctx: click.Context, file: str, url: str, username: str, email: str, headers: dict | None = None, token: str = None) -> str:
    """
    Request the map generation to the API
    :param ctx: Click context
    :param file: File path
    :param url: URL for the map generation API
    :param username: Username
    :param email: Email
    :param optional headers: Headers for the request
    :return: Map generation result
    """
    click.secho(f"Generating map for file: {file}")
    read_file(ctx, file)

    try:
        # Send the file to the map generation API
        with open(file, "rb") as f:
            response = httpx.post(
                url,
                headers=headers,
                files={"file": f},
                timeout=EXTENDED_TIMEOUT,
            )
        if response.status_code != HTTPStatus.OK:
            raise click.ClickException(
                f"Failed to generate map from file '{file}'. Server response: {response.text}"
            )

        # Parse response and construct output
        json_data = response.json()
        salt = json_data.get("salt")
        public_salt = json_data.get("public_salt")
        status_name = json_data.get("status_name")
        public_url = f"{PRODUCT_MAP_DOMAIN_URL}/app/public/{public_salt}"
        set_map_sharing_status(
            ctx,
            salt,
            is_shared=True,
            username=username,
            email=email,
            token=token,
        )
        click.secho(f"Map being processed for file '{file}'.", fg="blue")
        click.secho(f"Map can be visualized once the generation process finishes...", fg="blue")
        click.echo(json.dumps({
            "salt": salt,
            "public_url": public_url,
            "status_name": status_name
        }))

    except Exception as e:
        click.echo(json.dumps({
            "status": "error",
            "message": "An error occurred during map generation request.",
            "details": str(e)
        }))


def get_map_generation_status(token: str, salt: str):
    """
    Get the map generation status for the given salt
    :param token: Auth token
    :param salt: Map salt
    :return: Map generation status
    """
    try:
        response = httpx.get(
            f"{PRODUCT_MAP_API_DOMAIN_URL}/llm/upload/result/{salt}",
            headers={"Authorization": f"Bearer {token}"},
            timeout=EXTENDED_TIMEOUT,
        )

        if response.status_code != HTTPStatus.OK:
            # Send structured error response
            click.echo(json.dumps({
                "status": "error",
                "message": f"Failed to get map generation status for salt '{salt}'.",
                "details": response.text
            }))
        data = response.json()
        status_name = data.get("status_name")
        public_url = f"{PRODUCT_MAP_DOMAIN_URL}/app/public/{data.get('public_salt')}"

        # Send structured success response
        click.echo(json.dumps({
            "salt": salt,
            "public_url": public_url,
            "status_name": status_name
        }))

    except Exception as e:
        # Send structured error response for unexpected errors
        click.echo(json.dumps({
            "status": "error",
            "message": "An error occurred during map generation status retrieval.",
            "details": str(e)
        }))
